==> pan-0.11.0.tar.bz2: pan-0.11.0/pan/base/serverlist.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-config.h>

#include <pan/base/base-prefs.h>
#include <pan/base/debug.h>
#include <pan/base/log.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/util-file.h>

static GSList * serverlist = NULL;

static Server * _active = NULL;

/**
***  Private Utilities
**/

static gint
server_name_compare (gconstpointer a, gconstpointer b)
{
	const Server* sa = (const Server*) a;
	const Server* sb = (const Server*) b;
	return g_strcasecmp (sa->name, sb->name);
}

/**
***  Server Module
**/

static void
server_folders_load (void)
{
	Server * server;
	debug_enter ("server_folders_load");

       	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
	if (server == NULL) /* new user; create the folder server */
	{
		/* add the Folder Server */
		server = server_new ();
		server->address = g_strdup ("Mock Server for Folders");
		server->name = g_strdup (INTERNAL_SERVER_NAME);
		serverlist_add_server (server);
	}
	else /* server already exists; load its grouplist */
	{
		server_ensure_groups_loaded (server, SERVER_GROUPS_ALL);
	}

	if (server_get_named_group (server, PAN_SENDLATER) == NULL)
	{
		Group * group = group_new (server, PAN_SENDLATER);
		group->readable_name = g_strdup (_(PAN_SENDLATER));
		group->description = g_strdup (_("Articles/Mail queued for posting via the \"Send Later\" button"));
		group_set_is_folder (group, TRUE);
		server_add_groups (server, &group, 1, NULL, NULL);
		log_add (LOG_INFO, _("Created folder pan.sendlater"));
	}

	if (server_get_named_group (server, PAN_SENT) == NULL)
	{
		Group * group = group_new (server, PAN_SENT);
		group->readable_name = g_strdup (_(PAN_SENT));
		group->description = g_strdup (_("Articles/Mail posted before"));
		group_set_is_folder (group, TRUE);
		server_add_groups (server, &group, 1, NULL, NULL);
		log_add (LOG_INFO, _("Created folder pan.sent"));
	}

	debug_exit ("server_folders_load");
}

static Server*
server_fetch (const char *name)
{
	Server * server = NULL;
	gboolean has_section;
	gchar * buf;

	/* find out if we've got a gnome_config section for this server */
	buf = g_strdup_printf ("/Pan/%s/", name);
	gnome_config_push_prefix (buf);
	has_section = gnome_config_has_section (buf);
	replace_gstr (&buf, NULL);

	if (has_section)
	{
		server = server_new ();
		server->address = gnome_config_get_string ("Address");
		if (server->address == NULL)
		{
			pan_object_unref (PAN_OBJECT(server));
			server = NULL;
		}
		else
		{
			gchar * path;
			gchar * pch;
			
			/* make sure the server's config directory exists */
		       	path = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, name);
			directory_check (path);

			/* load the server from gnome_config */
			server->port =
				gnome_config_get_int ("Port=119");
			server->name =
				g_strdup(name);
			server->gen_msgid =
				gnome_config_get_int ("Generate_Msgid=0");
			server->need_auth =
				gnome_config_get_int ("Auth_Required=0");
			server->username =
				gnome_config_get_string ("Username");
			server->password =
				gnome_config_get_string ("Password");
			server->max_connections =
				gnome_config_get_int ("Max_Connections=2");
			server->idle_secs_before_timeout =
				gnome_config_get_int ("Idle_Timeout_Secs=180");
			server->online =
				gnome_config_get_bool ("Online=TRUE");
			server->reserve_connection_for_bodies =
				gnome_config_get_bool ("Reserve_Reader_Connection=TRUE");
			/* no good way to get a ulong from gnome_config? */
			server->last_newgroup_list_time = 1;
			pch = gnome_config_get_string ("Last_Grouplist_Update");
			if (is_nonempty_string(pch))
				server->last_newgroup_list_time = strtoul (pch, NULL, 10);

			/* cleanup */
			g_free (path);
			g_free (pch);
		}
	}
	gnome_config_pop_prefix ();

	return server;
}

/**
 * Load a list of servers from the user's ~/.gnome/Pan file
 * @return a GSList of Server's for each server found
 */
static GSList*
server_list_load (void)
{
	GSList * l = NULL;
	gchar * p = gnome_config_get_string ("/Pan/Servers/Names"); /*commas*/

	/* build all the servers in the list */
	if (p != NULL)
	{
		const char * walk = p;
		GString * str = g_string_new (NULL);

		/* create a server object for each server in the list */
		while (get_next_token_g_str (walk, ',', &walk, str))
		{
			Server * server = server_fetch (str->str);
			if (server != NULL)
				l = g_slist_insert_sorted (l,
				                           server,
				                           server_name_compare);
		}

		/* cleanup */
		g_string_free (str, TRUE);
		g_free (p);
	}

	return l;

}

static void
server_init_servers (void)
{
	static gboolean inited = FALSE;

	if (!inited)
	{
		inited = TRUE;
		serverlist = server_list_load ();
		server_folders_load ();
	}
}

void
serverlist_get_servers (GPtrArray * fillme)
{
	GSList * l;

	g_return_if_fail (fillme!=NULL);

	pan_g_ptr_array_reserve (fillme, g_slist_length(serverlist));
	for (l=serverlist; l!=NULL; l=l->next)
		g_ptr_array_add (fillme, l->data);
}

void
serverlist_shutdown (void)
{
	GSList * l;

	/* save the updated server info */
	serverlist_save ();

	/* close down the servers */
	l = serverlist;
	serverlist = NULL;
	g_slist_foreach (l, (GFunc)pan_object_unref, NULL);
	g_slist_free (l);
}

Server*
serverlist_get_active_server (void)
{
	return _active;
}

void
serverlist_set_active_server (Server * server)
{
	g_return_if_fail (server != NULL);

	_active = server;
	pan_callback_call (serverlist_get_server_activated_callback(), _active, NULL);
}

Server*
serverlist_get_named_server (const char* servername)
{
	GSList * l;
	Server * retval = NULL;
	debug_enter ("server_get_by_name");

	g_return_val_if_fail (is_nonempty_string(servername), NULL);

	server_init_servers ();

	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
		Server * s = SERVER(l->data);
		if (!pan_strcmp (s->name, servername))
			retval = s;
	}

	debug_exit ("server_get_by_name");
	return retval;
}

Group*
serverlist_get_named_folder (const gchar * foldername)
{
	Server * server;
		 
	/* sanity clause */
	g_return_val_if_fail (is_nonempty_string(foldername), NULL);
			 
	/* get the folder server... */
	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
	g_return_val_if_fail (server!=NULL, NULL);
					 
	/* get the folder... */
	return server_get_named_group (server, foldername);
}

static Server*
get_default_server (void)
{
	GSList * l;
	gchar * name;
	Server * retval = NULL;

	g_return_val_if_fail (serverlist!=NULL, NULL);

	/* look in the gnome config for a default */
	name = gnome_config_get_string ("/Pan/State/Server");
	if (is_nonempty_string (name))
		retval = serverlist_get_named_server (name);
	g_free (name);

	/* otherwise just use the first server */
	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
		Server * s = SERVER(l->data);
		if (strcmp (s->name, INTERNAL_SERVER_NAME))
			retval = s;
	}

	/* return what we've got */
	return retval;
}



/****
*****
*****  SERVER LOADING ON STARTUP
*****
****/

void
serverlist_init (void)
{
	Server * s;

	server_init_servers ();

	s = get_default_server ();
	if (s != NULL)
		serverlist_set_active_server (s);
}

void
serverlist_add_server (Server * server)
{
	g_return_if_fail (server!=NULL);

	server_init_servers ();

	serverlist = g_slist_insert_sorted (serverlist,
	                                    server,
	                                    server_name_compare);
}

void
serverlist_remove_server (Server * server)
{
	g_return_if_fail (server!=NULL);

	serverlist = g_slist_remove (serverlist, server);
	/* might be best to just let this leak.
	pan_object_unref (PAN_OBJECT(server)); */
}


void
serverlist_save (void)
{
	GString * buf = g_string_new (NULL);
	GString * servers = g_string_new (NULL);
	GSList *l = NULL;
	gchar * p;

	/* Out with the old */
	p = gnome_config_get_string ("/Pan/Servers/Names");
	if (p != NULL) {
		const char * pch = p;
		while (get_next_token_g_str(pch, ',', &pch, buf)) {
			g_string_prepend (buf, "/Pan/");
			gnome_config_clean_section (buf->str);
		}
		g_free (p);
	}
	gnome_config_sync ();

	/* In with the new */
	for (l=serverlist; l!=NULL; l=l->next)
	{
		char * pch;
		Server *s = SERVER(l->data);

		g_string_sprintf (buf, "/Pan/%s/Address", s->name);
		gnome_config_set_string (buf->str, s->address);

		g_string_sprintf (buf, "/Pan/%s/Port", s->name );
		gnome_config_set_int (buf->str, s->port);

		g_string_sprintf (buf, "/Pan/%s/Auth_Required", s->name);
		gnome_config_set_int (buf->str, s->need_auth);

		g_string_sprintf (buf, "/Pan/%s/Generate_Msgid", s->name);
		gnome_config_set_int (buf->str, s->gen_msgid);

		g_string_sprintf (buf, "/Pan/%s/Username", s->name);
		gnome_config_set_string (buf->str, s->username);

		g_string_sprintf (buf, "/Pan/%s/Password", s->name);
		gnome_config_set_string (buf->str, s->password);

		g_string_sprintf (buf, "/Pan/%s/Max_Connections", s->name);
		gnome_config_set_int (buf->str, s->max_connections);

		g_string_sprintf (buf, "/Pan/%s/Idle_Timeout_Secs", s->name);
		gnome_config_set_int (buf->str, s->idle_secs_before_timeout);

		g_string_sprintf (buf, "/Pan/%s/Online", s->name);
		gnome_config_set_bool (buf->str, s->online);

		/* no good way to store a ulong in gnome_config? */
		pch = g_strdup_printf ("%lu", s->last_newgroup_list_time);
		g_string_sprintf (buf,"/Pan/%s/Last_Grouplist_Update", s->name);
		gnome_config_set_string (buf->str, pch);
		g_free (pch);

		g_string_sprintf (buf, "/Pan/%s/Reserve_Reader_Connection", s->name);
		gnome_config_set_bool (buf->str, s->reserve_connection_for_bodies);

		if (servers->len)
			g_string_append_c (servers, ',');
		g_string_append (servers, s->name);
	}

	gnome_config_set_string ("/Pan/Servers/Names", servers->str);
	g_string_free (servers, TRUE);
	g_string_free (buf, TRUE);
	gnome_config_sync ();
}

#if 0
void
serverlist_activate_default_server (void)
{
	Server * s = get_default_server ();
	if (s != NULL) {
		_active = s;
		fire_server_activated (s);
	}
}
#endif


/**
***  Server Activated
**/

PanCallback*
serverlist_get_server_activated_callback (void)
{
	static PanCallback * cb = NULL;
	if (cb==NULL) cb = pan_callback_new ();
	return cb;
}

==> pan-0.11.2.91.tar.bz2: pan-0.11.2.91/pan/base/serverlist.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <pan/base/base-prefs.h>
#include <pan/base/debug.h>
#include <pan/base/log.h>
#include <pan/base/pan-config.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/server.h>
#include <pan/base/serverlist.h>
#include <pan/base/util-file.h>

static GSList * serverlist = NULL;

static Server * _active = NULL;

const gchar * PAN_SENT;
const gchar * PAN_SENDLATER;

/**
***  Private Utilities
**/

static gint
server_name_compare (gconstpointer a, gconstpointer b)
{
	const Server* sa = (const Server*) a;
	const Server* sb = (const Server*) b;
	return g_strcasecmp (server_get_name (sa), server_get_name (sb));
}

/**
***  Server Module
**/

static void
server_folders_load (void)
{
	Server * server;
	GPtrArray * groups = g_ptr_array_new ();
	debug_enter ("server_folders_load");


       	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
	if (server == NULL) /* new user; create the folder server */
	{
		/* add the Folder Server */
		server = server_new ();
		server->address = g_strdup ("Mock Server for Folders");
		server->name = g_strdup (INTERNAL_SERVER_NAME);
		serverlist_add_server (server);
	}
	else /* server already exists; load its grouplist */
	{
		server_ensure_groups_loaded (server, SERVER_GROUPS_ALL);
	}

	if (server_get_named_group (server, PAN_SENDLATER) == NULL)
	{
		Group * group = group_new (server, PAN_SENDLATER);
		group->readable_name = g_strdup (_(PAN_SENDLATER));
		group->description = g_strdup (_("Articles/Mail queued for posting via the \"Send Later\" button"));
		group_set_is_folder (group, TRUE);
		g_ptr_array_add (groups, group);
		log_add (LOG_INFO, _("Created folder pan.sendlater"));
	}

	if (server_get_named_group (server, PAN_SENT) == NULL)
	{
		Group * group = group_new (server, PAN_SENT);
		group->readable_name = g_strdup (_(PAN_SENT));
		group->description = g_strdup (_("Articles/Mail posted before"));
		group_set_is_folder (group, TRUE);
		g_ptr_array_add (groups, group);
		log_add (LOG_INFO, _("Created folder pan.sent"));
	}

	if (groups->len != 0)
		server_add_groups (server, (Group**)groups->pdata, groups->len, NULL, NULL);

	g_ptr_array_free (groups, TRUE);
	debug_exit ("server_folders_load");
}

static Server*
server_fetch (const char *name)
{
	Server * server = NULL;
	gboolean has_section;
	gchar * buf;

	/* find out if we've got a pan_config section for this server */
	buf = g_strdup_printf ("/Pan/%s/", name);
	pan_config_push_prefix (buf);
	has_section = pan_config_has_section (buf);
	replace_gstr (&buf, NULL);

	if (has_section)
	{
		server = server_new ();
		server->address = pan_config_get_string ("Address");
		if (server->address == NULL)
		{
			pan_object_unref (PAN_OBJECT(server));
			server = NULL;
		}
		else
		{
			gchar * path;
			gchar * pch;
			
			/* make sure the server's config directory exists */
		       	path = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, name);
			directory_check (path);

			/* load the server from pan_config */
			server->port =
				pan_config_get_int ("Port=119");
			server->name =
				g_strdup(name);
			server->gen_msgid =
				pan_config_get_int ("Generate_Msgid=0");
			server->need_auth =
				pan_config_get_int ("Auth_Required=0");
			server->username =
				pan_config_get_string ("Username");
			server->password =
				pan_config_get_string ("Password");
			server->max_connections =
				pan_config_get_int ("Max_Connections=2");
			server->idle_secs_before_timeout =
				pan_config_get_int ("Idle_Timeout_Secs=180");
			server->online =
				pan_config_get_bool ("Online=TRUE");
			server->reserve_connection_for_bodies =
				pan_config_get_bool ("Reserve_Reader_Connection=TRUE");
			/* no good way to get a ulong from pan_config? */
			server->last_newgroup_list_time = 1;
			pch = pan_config_get_string ("Last_Grouplist_Update");
			if (is_nonempty_string(pch))
				server->last_newgroup_list_time = strtoul (pch, NULL, 10);

			/* cleanup */
			g_free (path);
			g_free (pch);
		}
	}
	pan_config_pop_prefix ();

	return server;
}

/**
 * Load a list of servers from the user's ~/.gnome/Pan file
 * @return a GSList of Server's for each server found
 */
static GSList*
server_list_load (void)
{
	GSList * l = NULL;
	gchar * p = pan_config_get_string ("/Pan/Servers/Names"); /*commas*/

	/* build all the servers in the list */
	if (p != NULL)
	{
		const char * walk = p;
		GString * str = g_string_new (NULL);

		/* create a server object for each server in the list */
		while (get_next_token_g_str (walk, ',', &walk, str))
		{
			Server * server = server_fetch (str->str);
			if (server != NULL)
				l = g_slist_insert_sorted (l,
				                           server,
				                           server_name_compare);
		}

		/* cleanup */
		g_string_free (str, TRUE);
		g_free (p);
	}

	return l;

}

static void
server_init_servers (void)
{
	static gboolean inited = FALSE;

	if (!inited)
	{
		inited = TRUE;
		serverlist = server_list_load ();
		server_folders_load ();
	}
}

void
serverlist_get_servers (GPtrArray * fillme)
{
	GSList * l;

	g_return_if_fail (fillme!=NULL);

	pan_g_ptr_array_reserve (fillme, g_slist_length(serverlist));
	for (l=serverlist; l!=NULL; l=l->next)
		g_ptr_array_add (fillme, l->data);
}

void
serverlist_shutdown (void)
{
	GSList * l;

	/* save the updated server info */
	serverlist_save ();

	/* close down the servers */
	l = serverlist;
	serverlist = NULL;
	g_slist_foreach (l, (GFunc)pan_object_unref, NULL);
	g_slist_free (l);
}

Server*
serverlist_get_active_server (void)
{
	return _active;
}

void
serverlist_set_active_server (Server * server)
{
	g_return_if_fail (server != NULL);

	_active = server;
	pan_callback_call (serverlist_get_server_activated_callback(), _active, NULL);
}

Server*
serverlist_get_named_server (const char* servername)
{
	GSList * l;
	Server * retval = NULL;
	debug_enter ("server_get_by_name");

	g_return_val_if_fail (is_nonempty_string(servername), NULL);

	server_init_servers ();

	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
		Server * s = SERVER(l->data);
		if (!pan_strcmp (server_get_name (s), servername))
			retval = s;
	}

	debug_exit ("server_get_by_name");
	return retval;
}

Group*
serverlist_get_named_folder (const gchar * foldername)
{
	Server * server;
		 
	/* sanity clause */
	g_return_val_if_fail (is_nonempty_string(foldername), NULL);
			 
	/* get the folder server... */
	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
	g_return_val_if_fail (server!=NULL, NULL);
					 
	/* get the folder... */
	return server_get_named_group (server, foldername);
}

static Server*
get_default_server (void)
{
	GSList * l;
	gchar * name;
	Server * retval = NULL;

	g_return_val_if_fail (serverlist!=NULL, NULL);

	/* look in the gnome config for a default */
	name = pan_config_get_string ("/Pan/State/Server");
	if (is_nonempty_string (name))
		retval = serverlist_get_named_server (name);
	g_free (name);

	/* otherwise just use the first server */
	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
		Server * s = SERVER(l->data);
		if (strcmp (server_get_name (s), INTERNAL_SERVER_NAME))
			retval = s;
	}

	/* return what we've got */
	return retval;
}



/****
*****
*****  SERVER LOADING ON STARTUP
*****
****/

void
serverlist_init (void)
{
	Server * s;

	server_init_servers ();

	s = get_default_server ();
	if (s != NULL)
		serverlist_set_active_server (s);
}

void
serverlist_add_server (Server * server)
{
	g_return_if_fail (server!=NULL);

	server_init_servers ();

	serverlist = g_slist_insert_sorted (serverlist,
	                                    server,
	                                    server_name_compare);
	serverlist_save ();
}

void
serverlist_remove_server (Server * server)
{
	g_return_if_fail (server!=NULL);

	serverlist = g_slist_remove (serverlist, server);
	/* might be best to just let this leak.
	pan_object_unref (PAN_OBJECT(server)); */
}


void
serverlist_save (void)
{
	GString * servers = g_string_new (NULL);
	GSList *l = NULL;
	gchar * p;
	char buf[2048];

	/* Out with the old */
	p = pan_config_get_string ("/Pan/Servers/Names");
	if (p != NULL) {
		int str_len = 0;
		const char * str_begin = NULL;
		const char * walk = p;
		while (get_next_token_run (walk, ',', &walk, &str_begin, &str_len)) {
			g_snprintf (buf, sizeof(buf), "/Pan/%*.*s", str_len, str_len, str_begin);
			pan_config_clean_section (buf);
		}
		g_free (p);
	}
	pan_config_sync ();

	/* In with the new */
	for (l=serverlist; l!=NULL; l=l->next)
	{
		char val[128];
		Server *s = SERVER(l->data);
		const char * sn = server_get_name (s);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Address", sn);
		pan_config_set_string (buf, s->address);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Port", sn);
		pan_config_set_int (buf, s->port);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Auth_Required", sn);
		pan_config_set_int (buf, s->need_auth);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Generate_Msgid", sn);
		pan_config_set_int (buf, s->gen_msgid);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Username", sn);
		pan_config_set_string (buf, s->username);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Password", sn);
		pan_config_set_string (buf, s->password);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Max_Connections", sn);
		pan_config_set_int (buf, s->max_connections);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Idle_Timeout_Secs", sn);
		pan_config_set_int (buf, s->idle_secs_before_timeout);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Online", sn);
		pan_config_set_bool (buf, s->online);

		g_snprintf (buf, sizeof(buf),"/Pan/%s/Last_Grouplist_Update", sn);
		g_snprintf (val, sizeof(val), "%lu", s->last_newgroup_list_time);
		pan_config_set_string (buf, val);

		g_snprintf (buf, sizeof(buf), "/Pan/%s/Reserve_Reader_Connection", sn);
		pan_config_set_bool (buf, s->reserve_connection_for_bodies);

		if (servers->len)
			g_string_append_c (servers, ',');
		g_string_append (servers, server_get_name (s));
	}

	pan_config_set_string ("/Pan/Servers/Names", servers->str);
	g_string_free (servers, TRUE);
	pan_config_sync ();
}

/**
***  Server Activated
**/

PanCallback*
serverlist_get_server_activated_callback (void)
{
	static PanCallback * cb = NULL;
	if (cb==NULL) cb = pan_callback_new ();
	return cb;
}
